#ifndef PORTABLE_IO_HEADER
#define PORTABLE_IO_HEADER

///@file portable_io.h
///
///   This file exists to make most level 1 and level 2 i/o functions portable.
///   It also include directory reading functions and file control functions
///   so as to provide a single location to grab most system i/o operations
///   in a standard way.
///
///   Some other i/o portability declarations are also made here.

#include <stdlib.h>
#include <stdio.h>

#ifdef _MSC_VER
# include <process.h>
# include <io.h>
# include <direct.h>
# define  snprintf _snprintf
#define popen  _popen
#define pclose _pclose
#else
#include <cstddef>
# include <unistd.h>
# define  O_BINARY 0
#endif

#include <fcntl.h>

namespace cxxtls
{

template<class Iterator>
inline
Iterator
copy_end_of_line(Iterator output)
    ///  Copy the end of line sequence to a buffer given an iterator
    ///  and given the way in which the program is run.  If the program
    ///  is not run under windows, just output \n, but if it is run
    ///  under windows BUT is NOT run under a unix shell like bash, korn,
    ///  etc, then put out \\r\\n.
{
  #ifdef _MSC_VER

  //
  //  Note:  for this code to work the way you expect, you will need to make sure that
  //  the io stream is in the binary mode -- otherwise, you'll get \r's whether you want
  //  them or not.
  //
  //  To ensure that cout is in binary mode, put this in your main
  //
  //     #include <io.h>
  //
  //     main()
  //     {
  //        cout.flush();
  //        setmode(1, O_binary);
  //        ...
  //

  static bool initialized = false;
  static bool underBash   = false;

  if(!initialized)
  {
    initialized = true;

    char*  p = getenv("SHLVL");

    if(p)
    {
       // SHLVL is a bash, sh, csh, ksh flag to keep track of how
       // many subshells deep we are
    
       underBash = true;
    }

//    else
//    {
//      system("set");
//    }

  } // of initialization


  if(!underBash)
    *(output)++ = '\r';

  #endif

  *(output)++ = '\n';

  return output;
}

#ifdef _MSC_VER

 extern struct EOL_Sequence_t { char s[2]; }  EOL_Sequence;

#else

 extern struct EOL_Sequence_t { char s[1]; }  EOL_Sequence;

#endif

} // namespace cxxtls
#endif
