//
// Copyright 2015 Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.

#include <cxxtls/commandLineOptions.h>

namespace cxxtls
{

namespace CommandLine
{

    OptionDeclaration const *Options::optionDeclaration(std::string const &name) const
        /// @return null if the name is not a valid option or
        /// return a pointer to the OptionDeclaration object if it is.
    {
       std::vector<OptionDeclaration>::const_iterator f = optionInfo_.begin(), l = optionInfo_.end();

       while(f != l)
       {
           if(name == f->name_)
              return &*f;

          ++f;
       }

       return 0;
    }

    std::ostream &Options::printHelp(std::ostream &s) const
    {

        std::vector<OptionDeclaration>::const_iterator f = optionInfo_.begin(), l = optionInfo_.end();

        // compute the maximum size of the name

        size_t nameSize = 0;

        while(f != l)
        {
           if(f->name_.size() > nameSize)
              nameSize = f->name_.size();

           ++f;
        }

        nameSize += 2; 

        // now print the options in columnated format

        f = optionInfo_.begin();
        l = optionInfo_.end();

        std::string optionPrompt("Option");

        while(optionPrompt.size() < nameSize)
           optionPrompt += ' ';

        s << optionPrompt << "  " << "Parms?" << "  " << "Description\n";

        while(f != l)
        {
           std::string name = f->name_;

           while(name.size() < nameSize)
              name += ' ';

           s << "" << (f->mode_ ? "--" : " -" ) << name;
           
           s << char(f->parms_ <= 1 ? f->parms_+'0':'*') << "       " << f->help_<< '\n';
           ++f;
        }

        return s;
    }





    
std::vector<std::string> Options::emptyVector_;


void commandLineOptionsTestFunction()
{
   // no one should call this

   static char  const *testArgs[]={
                               "arg0",
                               "-f1",
                               "--F2",
                               "arg1",
                               "arg2"
                            };

   static OptionDeclaration optTable[] =
   {
      OptionDeclaration("-g", 0, 0, "help1"),
      OptionDeclaration("-f", 1, 0, "help2"),
      OptionDeclaration("-F", 2, 1, "help3"),
   };

   Options tmp((char**)testArgs, optTable, sizeof(optTable)/sizeof(optTable[0]));

}


}  // namespace CommandLine

}  // namespace cxxtls


