//
// Copyright 2002, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.
//
// Warning:  not all files in this directory structure are covered by the
// same copyright.  Some of them are part of the GNU source distribution
// and you must obey the GPL copyright for those files.

#include <cxxtls/cutset.h>
#include <cxxtls/strtool.h>
#include <cxxtls/foreach.h>

#include <ctype.h>
#include <stdio.h>

using namespace std;

namespace cxxtls
{

CutSet::
CutSet(std::string const &spec)
:  maxPos_(0)
{

  string::const_iterator begin=spec.begin(), end=spec.end();

  while(begin != end)
  {
    char c = *begin++;

    if(isdigit(c))
    {
      string number;

      while(isdigit(c))
      {
        number += c;

        if(begin == end)
        {
          c = 0;

          break;
        }
        else
          c = *begin++;
      }

      int value=0;

      sscanf(number.c_str(), "%d", &value);

      if(value < 1)
        break; // silently fail

      if(   (c == '-'  )
         && (begin != end)
        )
      {
        // we have a character range

        c = *begin++;

        if(isdigit(c))
        {
          number.resize(0);

          while(isdigit(c))
          {
            number += c;
          
            if(begin == end)
            {
              c = 0;
          
              break;
            }
            else
              c = *begin++;
          }

          int value2=0;


          sscanf(number.c_str(), "%d", &value2);

         if(value2 < 1)
           break; // silently fail

         addRange(value,value2);

        }

      }
      else
        addCharPos(value);


    }
    else
    if(c == ',')
      continue;  // char ranges are separated by commas
    else
     break; // silently fail

  }



}

string
CutSet::
fetchCharacters (string const &input) const
{            
  size_t inputSize = input.size();

  string rv;

  RangeList::const_iterator begin, end;
  

  for(begin = ranges_.begin(), end = ranges_.end(); begin != end ; ++begin)
  {

    size_t start  = size_t(begin->first);
    size_t length = size_t(begin->second - begin->first + 1);

    if(start < inputSize)
    {
      size_t maxLength = inputSize - start;

      if(length > maxLength)
        length = maxLength;

      char const *data = input.data() + start;

      if (start + length >= inputSize)
      {
          if (start >= inputSize)
          {
              length = 0;
              start = 0;
              data = input.data();
          }
          else
          {
              length = inputSize - start;
          }
      }

      char const *end  = data + length;

      while(data != end)
      {
        rv += *data++;
      }
    }

  }

 return rv;


}

std::string
CutSet::
fetchFields (std::vector<std::string> const &input, char delim) const
{            
  size_t inputSize = input.size();

  // inputSize is now the count of strings stored in the array "input"

  string rv;

  RangeList::const_iterator begin, end;

  bool firstTimeThrough = true;

  for(begin = ranges_.begin(), end = ranges_.end(); begin != end ; ++begin)
  {

    size_t start  = size_t(begin->first);
    size_t length = size_t(begin->second - begin->first + 1);
    size_t end = start + length;

    // start and end are array indexes into input[]

    if(start < inputSize)
    {


        while(start < end && start < inputSize)
        {
           if(firstTimeThrough)
              firstTimeThrough = false;
           else
              if(delim)
              {
                  rv += delim;
              }

           rv += input[start];
           ++start;
        }
    }

  }

 return rv;


}


string
CutSet::
fetchDelimitedFields (string const &input_string, char delim) const
{            
  std::vector<std::string> input;

  size_t startSize = input_string.size() / 4;

  if(startSize)
       input.reserve( startSize );

  StrTool::splitOnDelimiter(input_string, input, StrTool::Is_Delim(delim));

  return fetchFields(input, delim);


}


string
CutSet::
fetchWords (string const &input_string) const
{            
  std::vector<std::string> input;

  size_t startSize = input_string.size() / 4;

  if(startSize)
       input.reserve( startSize );

  StrTool::stringlist_t words;

  StrTool::parse_words(input_string, &words);

  CXXTLS_FOREACH(string word, words)
  {
      input.push_back(word);
  }

  return fetchFields(input,' ');


}



} // namespace cxxtls
